/* ignuit - Educational software for the GNOME, following the Leitner
 * flash-card system.
 *
 * Copyright (C) 2008 Timothy Musson
 *
 * <trmusson@ihug.co.nz>
 * http://homepages.ihug.co.nz/~trmusson/programs.html#ignuit
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <config.h>
#include <gnome.h>
#include <glade/glade.h>

#include "main.h"
#include "file.h"
#include "prefs.h"
#include "dialog-preferences.h"


typedef struct {

    File                *file;
    Prefs               *prefs;

    GladeXML            *gui;
    GtkWidget           *window;

    GtkWidget           *btn_color_fg;
    GtkWidget           *btn_color_bg;
    GtkWidget           *btn_color_bg_known;
    GtkWidget           *btn_color_bg_unknown;
    GtkWidget           *btn_color_bg_end;

    GtkWidget           *spin_0;
    GtkWidget           *spin_1;
    GtkWidget           *spin_2;
    GtkWidget           *spin_3;
    GtkWidget           *spin_4;
    GtkWidget           *spin_5;
    GtkWidget           *spin_6;
    GtkWidget           *spin_7;

} DialogData;


static DialogData *dialog = NULL;


static void
set_color_buttons (DialogData *data)
{
    gtk_color_button_set_color (GTK_COLOR_BUTTON(data->btn_color_fg),
        prefs_get_color (data->prefs, COLOR_CARD_FG));
    gtk_color_button_set_color (GTK_COLOR_BUTTON(data->btn_color_bg),
        prefs_get_color (data->prefs, COLOR_CARD_BG));
    gtk_color_button_set_color (GTK_COLOR_BUTTON(data->btn_color_bg_known),
        prefs_get_color (data->prefs, COLOR_CARD_BG_KNOWN));
    gtk_color_button_set_color (GTK_COLOR_BUTTON(data->btn_color_bg_unknown),
        prefs_get_color (data->prefs, COLOR_CARD_BG_UNKNOWN));
    gtk_color_button_set_color (GTK_COLOR_BUTTON(data->btn_color_bg_end),
        prefs_get_color (data->prefs, COLOR_CARD_BG_END));
}


static void
set_spin_buttons (DialogData *data)
{
    gtk_spin_button_set_value (GTK_SPIN_BUTTON(data->spin_0),
        prefs_get_schedule (data->prefs, GROUP_NEW));
    gtk_spin_button_set_value (GTK_SPIN_BUTTON(data->spin_1),
        prefs_get_schedule (data->prefs, GROUP_1));
    gtk_spin_button_set_value (GTK_SPIN_BUTTON(data->spin_2),
        prefs_get_schedule (data->prefs, GROUP_2));
    gtk_spin_button_set_value (GTK_SPIN_BUTTON(data->spin_3),
        prefs_get_schedule (data->prefs, GROUP_3));
    gtk_spin_button_set_value (GTK_SPIN_BUTTON(data->spin_4),
        prefs_get_schedule (data->prefs, GROUP_4));
    gtk_spin_button_set_value (GTK_SPIN_BUTTON(data->spin_5),
        prefs_get_schedule (data->prefs, GROUP_5));
    gtk_spin_button_set_value (GTK_SPIN_BUTTON(data->spin_6),
        prefs_get_schedule (data->prefs, GROUP_6));
    gtk_spin_button_set_value (GTK_SPIN_BUTTON(data->spin_7),
        prefs_get_schedule (data->prefs, GROUP_7));
}


static void
cb_font_small (GtkFontButton *widget, DialogData *data)
{
    prefs_set_font_small (data->prefs, gtk_font_button_get_font_name (widget));
}


static void
cb_font_large (GtkFontButton *widget, DialogData *data)
{
    prefs_set_font_large (data->prefs, gtk_font_button_get_font_name (widget));
}


static void
cb_color_fg (GtkColorButton *widget, DialogData *data)
{
    GdkColor color;

    gtk_color_button_get_color (widget, &color);
    prefs_set_color_gdk (data->prefs, COLOR_CARD_FG, &color);
}


static void
cb_color_bg (GtkColorButton *widget, DialogData *data)
{
    GdkColor color;

    gtk_color_button_get_color (widget, &color);
    prefs_set_color_gdk (data->prefs, COLOR_CARD_BG, &color);
}


static void
cb_color_bg_known (GtkColorButton *widget, DialogData *data)
{
    GdkColor color;

    gtk_color_button_get_color (widget, &color);
    prefs_set_color_gdk (data->prefs, COLOR_CARD_BG_KNOWN, &color);
}


static void
cb_color_bg_unknown (GtkColorButton *widget, DialogData *data)
{
    GdkColor color;

    gtk_color_button_get_color (widget, &color);
    prefs_set_color_gdk (data->prefs, COLOR_CARD_BG_UNKNOWN, &color);
}


static void
cb_color_bg_end (GtkColorButton *widget, DialogData *data)
{
    GdkColor color;

    gtk_color_button_get_color (widget, &color);
    prefs_set_color_gdk (data->prefs, COLOR_CARD_BG_END, &color);
}


static void
cb_restore_colors (GtkWidget *widget, DialogData *data)
{
    prefs_csvstr_to_colors (data->prefs, DEFAULT_CARD_COLORS);
    set_color_buttons (data);
}


static void
cb_schedule_changed (GtkWidget *widget, DialogData *data)
{
    Group group;
    gint days;

    if (widget == data->spin_0) {
        group = GROUP_NEW;
    }
    else if (widget == data->spin_1) {
        group = GROUP_1;
    }
    else if (widget == data->spin_2) {
        group = GROUP_2;
    }
    else if (widget == data->spin_3) {
        group = GROUP_3;
    }
    else if (widget == data->spin_4) {
        group = GROUP_4;
    }
    else if (widget == data->spin_5) {
        group = GROUP_5;
    }
    else if (widget == data->spin_6) {
        group = GROUP_6;
    }
    else if (widget == data->spin_7) {
        group = GROUP_7;
    }
    else {
        g_assert_not_reached();
    }

    days = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(widget));

    prefs_set_schedule (data->prefs, group, days);
}


static void
cb_restore_schedules (GtkWidget *widget, DialogData *data)
{
    prefs_csvstr_to_schedules (data->prefs, DEFAULT_SCHEDULES);
    set_spin_buttons (data);
}


static void
cb_destroy (GtkWidget *widget, DialogData *data)
{
    g_object_unref (G_OBJECT(dialog->gui));
    g_free (dialog);

    dialog = NULL;
}


static void
cb_close (GtkWidget *widget, DialogData *data)
{
    gtk_widget_destroy (data->window);
}


void
dialog_preferences (Ignuit *ig)
{
    GtkWidget *btn_close;
    GtkWidget *btn_font_small;
    GtkWidget *btn_font_large;
    GtkWidget *btn_restore_colors, *btn_restore_schedules;
    const gchar *font;
    gchar *gladefile;


    if (dialog != NULL) {
        gtk_window_present (GTK_WINDOW(dialog->window));
        return;
    }

    gladefile = gnome_program_locate_file (ig->program,
        GNOME_FILE_DOMAIN_APP_DATADIR, F_GLADE_PREFERENCES, TRUE, NULL);

    if (gladefile == NULL) {
        g_warning ("Can't find file: %s\n", F_GLADE_PREFERENCES);
        return;
    }

    dialog = g_new0 (DialogData, 1);
    dialog->gui = glade_xml_new (gladefile, NULL, NULL);
    g_free (gladefile);

    dialog->file = ig->file;
    dialog->prefs = ig->prefs;

    dialog->window = glade_xml_get_widget (dialog->gui, "dialog");

    btn_font_small = glade_xml_get_widget (dialog->gui, "btn_font_small");
    btn_font_large = glade_xml_get_widget (dialog->gui, "btn_font_large");
    dialog->btn_color_fg = glade_xml_get_widget (dialog->gui, "btn_color_fg");
    dialog->btn_color_bg = glade_xml_get_widget (dialog->gui, "btn_color_bg");
    dialog->btn_color_bg_known = glade_xml_get_widget (dialog->gui, "btn_color_bg_known");
    dialog->btn_color_bg_unknown = glade_xml_get_widget (dialog->gui, "btn_color_bg_unknown");
    dialog->btn_color_bg_end = glade_xml_get_widget (dialog->gui, "btn_color_bg_end");
    btn_restore_colors = glade_xml_get_widget (dialog->gui, "btn_restore_colors");

    dialog->spin_0 = glade_xml_get_widget (dialog->gui, "spin_0");
    dialog->spin_1 = glade_xml_get_widget (dialog->gui, "spin_1");
    dialog->spin_2 = glade_xml_get_widget (dialog->gui, "spin_2");
    dialog->spin_3 = glade_xml_get_widget (dialog->gui, "spin_3");
    dialog->spin_4 = glade_xml_get_widget (dialog->gui, "spin_4");
    dialog->spin_5 = glade_xml_get_widget (dialog->gui, "spin_5");
    dialog->spin_6 = glade_xml_get_widget (dialog->gui, "spin_6");
    dialog->spin_7 = glade_xml_get_widget (dialog->gui, "spin_7");
    btn_restore_schedules = glade_xml_get_widget (dialog->gui,
        "btn_restore_schedules");

    btn_close = glade_xml_get_widget (dialog->gui, "btn_close");

    set_color_buttons (dialog);
    set_spin_buttons (dialog);

    font = prefs_get_font_small (dialog->prefs);
    if (font != NULL) {
        gtk_font_button_set_font_name (GTK_FONT_BUTTON(btn_font_small), font);
    }

    font = prefs_get_font_large (dialog->prefs);
    if (font != NULL) {
        gtk_font_button_set_font_name (GTK_FONT_BUTTON(btn_font_large), font);
    }

    g_signal_connect (G_OBJECT(dialog->window), "destroy",
        G_CALLBACK(cb_destroy), dialog);

    g_signal_connect (G_OBJECT(btn_font_small), "font-set",
        G_CALLBACK(cb_font_small), dialog);
    g_signal_connect (G_OBJECT(btn_font_large), "font-set",
        G_CALLBACK(cb_font_large), dialog);
    g_signal_connect (G_OBJECT(dialog->btn_color_fg), "color-set",
        G_CALLBACK(cb_color_fg), dialog);
    g_signal_connect (G_OBJECT(dialog->btn_color_bg), "color-set",
        G_CALLBACK(cb_color_bg), dialog);
    g_signal_connect (G_OBJECT(dialog->btn_color_bg_known), "color-set",
        G_CALLBACK(cb_color_bg_known), dialog);
    g_signal_connect (G_OBJECT(dialog->btn_color_bg_unknown), "color-set",
        G_CALLBACK(cb_color_bg_unknown), dialog);
    g_signal_connect (G_OBJECT(dialog->btn_color_bg_end), "color-set",
        G_CALLBACK(cb_color_bg_end), dialog);

    g_signal_connect (G_OBJECT(btn_restore_colors), "clicked",
        G_CALLBACK(cb_restore_colors), dialog);

    g_signal_connect (G_OBJECT(dialog->spin_0), "value-changed",
        G_CALLBACK(cb_schedule_changed), dialog);
    g_signal_connect (G_OBJECT(dialog->spin_1), "value-changed",
        G_CALLBACK(cb_schedule_changed), dialog);
    g_signal_connect (G_OBJECT(dialog->spin_2), "value-changed",
        G_CALLBACK(cb_schedule_changed), dialog);
    g_signal_connect (G_OBJECT(dialog->spin_3), "value-changed",
        G_CALLBACK(cb_schedule_changed), dialog);
    g_signal_connect (G_OBJECT(dialog->spin_4), "value-changed",
        G_CALLBACK(cb_schedule_changed), dialog);
    g_signal_connect (G_OBJECT(dialog->spin_5), "value-changed",
        G_CALLBACK(cb_schedule_changed), dialog);
    g_signal_connect (G_OBJECT(dialog->spin_6), "value-changed",
        G_CALLBACK(cb_schedule_changed), dialog);
    g_signal_connect (G_OBJECT(dialog->spin_7), "value-changed",
        G_CALLBACK(cb_schedule_changed), dialog);

    g_signal_connect (G_OBJECT(btn_restore_schedules), "clicked",
        G_CALLBACK(cb_restore_schedules), dialog);

    g_signal_connect (G_OBJECT(btn_close), "clicked",
        G_CALLBACK(cb_close), dialog);


    gtk_window_set_transient_for (GTK_WINDOW(dialog->window),
        GTK_WINDOW(ig->app));

    gtk_window_set_modal (GTK_WINDOW(dialog->window), FALSE);

    gtk_widget_show_all (dialog->window);
}

