/* ignuit - Educational software for the GNOME, following the Leitner
 * flash-card system.
 *
 * Copyright (C) 2008 Timothy Musson
 *
 * <trmusson@ihug.co.nz>
 * http://homepages.ihug.co.nz/~trmusson/programs.html#ignuit
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <config.h>
#include <gnome.h>
#include <glade/glade.h>

#include "main.h"
#include "file.h"
#include "app-window.h"
#include "dialog-properties.h"


typedef struct {

    File                *file;

    GladeXML            *gui;
    GtkWidget           *window;
    GtkWidget           *parent;

    GtkTreeView         *treev;
    GtkTreeStore        *ts;

    GtkWidget           *entry_title;
    GtkWidget           *entry_author;
    GtkWidget           *entry_comment;

    CardStyle           style;

} DialogData;


static void
cb_style (GtkWidget *widget, DialogData *data)
{
    data->style = gtk_combo_box_get_active (GTK_COMBO_BOX(widget));
}


static void
cb_destroy (GtkWidget *widget, DialogData *data)
{
    g_object_unref (G_OBJECT(data->gui));
    g_free (data);
}


static gboolean
properties_changed (DialogData *data)
{
    return
        strcmp (file_get_title (data->file),
            gtk_entry_get_text (GTK_ENTRY(data->entry_title))) ||
        strcmp (file_get_author (data->file),
            gtk_entry_get_text (GTK_ENTRY(data->entry_author))) ||
        strcmp (file_get_comment (data->file),
            gtk_entry_get_text (GTK_ENTRY(data->entry_comment))) ||
        file_get_style (data->file) != data->style;
}


static void
cb_close (GtkWidget *widget, DialogData *data)
{
    if (properties_changed (data)) {

        file_set_title (data->file,
            gtk_entry_get_text (GTK_ENTRY(data->entry_title)));

        app_window_update_category_root_title (data->treev, data->ts,
            file_get_title (data->file));

        file_set_author (data->file,
            gtk_entry_get_text (GTK_ENTRY(data->entry_author)));
        file_set_comment (data->file,
            gtk_entry_get_text (GTK_ENTRY(data->entry_comment)));

        file_set_style (data->file, data->style);

        file_set_changed (data->file, TRUE);
    }

    gtk_widget_destroy (data->window);
}


void
dialog_properties (Ignuit *ig, GtkTreeView *treev, GtkTreeStore *ts)
{
    DialogData *dialog;
    GtkWidget *combo_style;
    GtkWidget *btn_close;
    gchar *gladefile;


    gladefile = gnome_program_locate_file (ig->program,
        GNOME_FILE_DOMAIN_APP_DATADIR, F_GLADE_PROPERTIES, TRUE, NULL);

    if (gladefile == NULL) {
        g_warning ("Can't find file: %s\n", F_GLADE_PROPERTIES);
        return;
    }

    dialog = g_new0 (DialogData, 1);
    dialog->gui = glade_xml_new (gladefile, NULL, NULL);
    g_free (gladefile);

    dialog->file = ig->file;
    dialog->parent = ig->app;

    dialog->treev = treev;
    dialog->ts = ts;

    dialog->window = glade_xml_get_widget (dialog->gui, "dialog");
    dialog->entry_title = glade_xml_get_widget (dialog->gui, "entry_title");
    dialog->entry_author = glade_xml_get_widget (dialog->gui, "entry_author");
    dialog->entry_comment = glade_xml_get_widget (dialog->gui, "entry_comment");
    combo_style = glade_xml_get_widget (dialog->gui, "combo_style");
    combo_style = glade_xml_get_widget (dialog->gui, "combo_style");
    btn_close = glade_xml_get_widget (dialog->gui, "btn_close");


    gtk_entry_set_text (GTK_ENTRY(dialog->entry_title),
        file_get_title (dialog->file));
    gtk_entry_set_text (GTK_ENTRY(dialog->entry_author),
        file_get_author (dialog->file));
    gtk_entry_set_text (GTK_ENTRY(dialog->entry_comment),
        file_get_comment (dialog->file));

    gtk_combo_box_append_text (GTK_COMBO_BOX(combo_style),
        _("Head Words"));
    gtk_combo_box_append_text (GTK_COMBO_BOX(combo_style),
        _("Head Words and Examples"));
    gtk_combo_box_append_text (GTK_COMBO_BOX(combo_style),
        _("Head Words and Sentences"));
    gtk_combo_box_append_text (GTK_COMBO_BOX(combo_style),
        _("Sentences"));

    dialog->style = file_get_style (dialog->file);
    gtk_combo_box_set_active (GTK_COMBO_BOX(combo_style), dialog->style);

    g_signal_connect (G_OBJECT(dialog->window), "destroy",
        G_CALLBACK(cb_destroy), dialog);

    g_signal_connect (G_OBJECT(combo_style), "changed",
        G_CALLBACK(cb_style), dialog);

    g_signal_connect (G_OBJECT(btn_close), "clicked",
        G_CALLBACK(cb_close), dialog);


    gtk_window_set_transient_for (GTK_WINDOW(dialog->window),
        GTK_WINDOW(ig->app));

    gtk_window_set_modal (GTK_WINDOW(dialog->window), FALSE);

    gtk_widget_show_all (dialog->window);
}

