/* file.h
 */


#ifndef HAVE_FILE_H
#define HAVE_FILE_H


#include "card.h"


typedef enum {
    CARD_STYLE_HEADWORDS = 0,
    CARD_STYLE_HEADWORDS_AND_EXAMPLES,
    CARD_STYLE_HEADWORDS_AND_SENTENCES,
    CARD_STYLE_SENTENCES
} CardStyle;


#define DEFAULT_FILE_JUSTIFY    GTK_JUSTIFY_CENTER


typedef struct _File File;


File*           file_new (void);
void            file_free (File *f, gboolean free_cards);

void            file_set_changed (File *f, gboolean changed);
gboolean        file_get_changed (File *f);

void            file_set_filename (File *f, const gchar *fname);
void            file_set_title (File *f, const gchar *title);
void            file_set_author (File *f, const gchar *author);
void            file_set_comment (File *f, const gchar *comment);

const gchar*    file_get_filename (File *f);
const gchar*    file_get_title (File *f);
const gchar*    file_get_author (File *f);
const gchar*    file_get_comment (File *f);

void            file_set_style (File *f, CardStyle style);
CardStyle       file_get_style (File *f);

void            file_set_current_category (File *f, Category *cat);
Category*       file_get_current_category (File *f);

GList*          file_add_category (File *f, Category *cat);
GList*          file_remove_category (File *f, Category *cat);
GList*          file_get_categories (File *f);
GList*          file_get_current_category_cards (File *f);

void            file_add_loaded_card (File *f, Card *c);
void            file_add_card (File *f, Category *cat, Card *c);
void            file_remove_card (File *f, Card *c);
GList*          file_get_cards (File *f);
guint           file_get_n_cards (File *f);

void            file_set_current_item (File *f, GList *item);
GList*          file_incr_current_item (File *f, gboolean wrap);
GList*          file_decr_current_item (File *f, gboolean wrap);
GList*          file_get_current_item (File *f);
Card*           file_get_current_card (File *f);

void            file_reset_statistics (File *f);

void            file_check_expired (File *f);


#endif  /* HAVE_FILE_H */

