/* prefs.h
 */


#ifndef HAVE_PREFS_H
#define HAVE_PREFS_H


#define PREF_APP_WIDTH              "/apps/ignuit/app_width"
#define PREF_APP_HEIGHT             "/apps/ignuit/app_height"
#define PREF_CATEGORY_PANE_WIDTH    "/apps/ignuit/category_pane_width"
#define PREF_QUIZ_WIDTH             "/apps/ignuit/quiz_width"
#define PREF_QUIZ_HEIGHT            "/apps/ignuit/quiz_height"
#define PREF_CARD_COLUMN_FRONT_WIDTH "/apps/ignuit/card_column_front_width"
#define PREF_STICKY_FLIPS           "/apps/ignuit/sticky_flips"
#define PREF_FONT_SMALL             "/apps/ignuit/font_small"
#define PREF_FONT_LARGE             "/apps/ignuit/font_large"
#define PREF_CARD_COLORS            "/apps/ignuit/card_colors"
#define PREF_SCHEDULES              "/apps/ignuit/schedules"
#define PREF_WORKDIR                "/apps/ignuit/workdir"

#define DEFAULT_APP_WIDTH           600
#define DEFAULT_APP_HEIGHT          300
#define DEFAULT_CATEGORY_PANE_WIDTH 225
#define DEFAULT_QUIZ_WIDTH          450
#define DEFAULT_QUIZ_HEIGHT         250
#define DEFAULT_CARD_COLUMN_FRONT_WIDTH   200
#define DEFAULT_SCHEDULES           "1,3,7,20,55,145,400,1000"
#define DEFAULT_CARD_COLORS         "#000000,#EFF2D3,#C5D2C8,#E0B6AF,#EED680"
#define DEFAULT_FONT_SMALL          "Sans 11"
#define DEFAULT_FONT_LARGE          "Sans 22"


typedef struct _Prefs Prefs;


typedef enum {
    COLOR_CARD_FG = 0,
    COLOR_CARD_BG,
    COLOR_CARD_BG_KNOWN,
    COLOR_CARD_BG_UNKNOWN,
    COLOR_CARD_BG_END,
    N_CARD_COLORS
} Color;


Prefs*          prefs_load (void);
void            prefs_free (Prefs *p);

Group           prefs_get_schedule (Prefs *p, Group g);
void            prefs_set_schedule (Prefs *p, Group g, gint days);
void            prefs_csvstr_to_schedules (Prefs *p, const gchar *csvstr);

void            prefs_set_sticky_flips (Prefs *p, gboolean sticky);
gboolean        prefs_get_sticky_flips (Prefs *p);

void            prefs_set_category_pane_width (Prefs *p, gint width);
gint            prefs_get_category_pane_width (Prefs *p);

void            prefs_set_color (Prefs *p, Color which, const gchar *spec);
void            prefs_set_color_gdk (Prefs *p, Color which, GdkColor *color);
GdkColor*       prefs_get_color (Prefs *p, Color which);
void            prefs_csvstr_to_colors (Prefs *p, const gchar *csvstr);

void            prefs_set_app_size (Prefs *p, gint width, gint height);
gint            prefs_get_app_width (Prefs *p);
gint            prefs_get_app_height (Prefs *p);

void            prefs_set_card_column_front_width (Prefs *p, gint width);
gint            prefs_get_card_column_front_width (Prefs *p);

void            prefs_set_quiz_size (Prefs *p, gint width, gint height);
gint            prefs_get_quiz_width (Prefs *p);
gint            prefs_get_quiz_height (Prefs *p);

void            prefs_set_font_small (Prefs *p, const gchar *fontname);
const gchar*    prefs_get_font_small (Prefs *p);

void            prefs_set_font_large (Prefs *p, const gchar *fontname);
const gchar*    prefs_get_font_large (Prefs *p);

void            prefs_set_workdir (Prefs *p, const gchar *dirname);
const gchar*    prefs_get_workdir (Prefs *p);


#endif /* HAVE_PREFS_H */

